@@ -1,11 +1,18 @@
-
 import logging
 import requests
 import xmltodict
 
 _LOGGER = logging.getLogger(__name__)
 
-class HttpVLC():
+class InvalidCredentials(Exception):
+    """Exception related to Invalid Credntials"""
+    pass
+
+class RequestFailed(Exception):
+    """Exception related to an Invalid Request"""
+    pass
+
+class HttpVLC:
     def __init__(self, host=None, username=None, password=None):
         self.host = host
         self.username = username or ''
@@ -21,9 +28,9 @@ class HttpVLC():
         if request.status_code == 200:
             return True
         elif request.status_code == 401:
-             raise('Unathorized! The provided username or password were incorrect')
+             raise InvalidCredentials("Unathorized! The provided username or password were incorrect")
         else:
-            raise f"Query failed, response code: {req.status_code} Full message: {req.text}"
+            raise RequestFailed(f"Query failed, response code: {request.status_code} Full message: {request.text}")
 
     def fetch_playlist(self):
         url = f"{self.host}/requests/playlist.xml"
@@ -38,34 +45,15 @@ class HttpVLC():
             data = xmltodict.parse(request.text, process_namespaces=True).get("root")
             return data
         except (requests.exceptions.ConnectionError, requests.exceptions.Timeout) as error:
-            raise f"The VLC Server is unreachable. Errorr code: {error}"
+            raise Exception(f"The VLC Server is unreachable. Errorr code: {error}")
 
     def fetch_data(self, command=None):
         return self.fetch_status(command)
 
-
-    def parse_data(self, command=None):
+    def parse_data(self, command=None, option='state', in_information=False):
         response_data = self.fetch_data(command=command)
-        # Root Elements
-        self._data['fullscreen'] = True if response_data['fullscreen'] == 'true' else False
-        self._data['aspectratio'] = response_data.get('aspectratio') or None
-        self._data['audiodelay'] = response_data['audiodelay']
-        self._data['apiversion'] = response_data['apiversion']
-        self._data['currentplid'] = response_data['currentplid']
-        self._data['time'] = response_data['time']
-        self._data['volume'] = response_data['volume']
-        self._data['length'] = response_data['length']
-        self._data['random'] = True if response_data['random'] == 'true' else False
-        self._data['state'] = response_data['state']
-        self._data['version'] = response_data['version']
-        self._data['position'] = response_data['position']
-        self._data['repeat'] = response_data['repeat']
-        self._data['subtitledelay'] = response_data['subtitledelay']
-        self._data['rate'] = response_data['rate']
-        self._data['loop'] = response_data['loop']
-        self._data['track_number'] = response_data['loop']
-        self._data['loop'] = response_data['loop']
-        self._data['loop'] = response_data['loop']
+        if not in_information:
+            return response_data.get(option) or None
 
         # Information Elements
         if 'information' in response_data:
@@ -73,80 +61,90 @@ class HttpVLC():
                 for info in response_data['information']['category']:
                     if info.get('@name') == 'meta':
                         for elem in info['info']:
-                            self._data[elem['@name']] = elem['#text']
+                            if elem['@name'] == option:
+                                return elem['#text']
             except:
                 pass
-        #self._data['album'] = response_data['information']['category'][''] or None
-
         return self._data
 
+    def update_data(self):
+        self.parse_data()
+
     def media_artist(self):
         """Return Current Artist playing"""
-        return self._data.get('artist') or None
+        return self.parse_data(option='artist', in_information=True) or None
 
     def is_fullscreen(self):
         """Return if VLC is in fullscreen"""
-        return self._data['fullscreen']
+        return True if self.parse_data(option='fullscreen') == 'true' else False
 
     def aspect_ratio(self):
         """Return Aspect Ratio of media playing"""
-        return self._data['aspectratio']
+        return self.parse_data(option='aspectratio')
+
+    def current_playlist_id(self):
+        """Return Current playlist id"""
+        return self.parse_data(option='currentplid')
 
     def audio_delay(self):
         """Return Aspect Ratio of media playing"""
-        return self._data['audiodelay']
+        return self.parse_data(option='audiodelay')
 
     def api_version(self):
         """Return API Version of VLC Server"""
-        return self._data['apiversion']
+        return self.parse_data(option='apiversion')
+
+    def version(self):
+        """Return Version of VLC Server"""
+        return self.parse_data(option='version')
 
     def media_time(self):
         """Return how long the media file is"""
-        return self._data['time']
+        return self.parse_data(option='time')
 
     def volume(self):
         """Return the volume of the media playing"""
-        return self._data['volume']
+        return self.parse_data(option='volume')
 
     def media_length(self):
         """Return the length of the media in seconds"""
-        return self._data['length']
+        return self.parse_data(option='length')
 
     def is_random(self):
         """Return if shuffle is on or off"""
-        return self._data['random']
+        return True if self.parse_data(option='random') == 'true' else False
 
     def rate(self):
         """Return the rate"""
-        return self._data['rate']
+        return self.parse_data(option='rate')
 
     def state(self):
         """Return the state of the media file"""
-        return self._data['state']
+        return self.parse_data(option='state')
 
     def is_looped(self):
         """Return if VLC is set on loop"""
-        return self._data['loop']
+        return self.parse_data(option='loop')
 
     def position(self):
         """Return the position of the playback """
-        return self._data['position']
+        return self.parse_data(option='position')
 
     def is_on_repeat(self):
         """Return if playback is set on repeat """
-        return self._data['repeat']
+        return self.parse_data(option='repeat')
 
     def album(self):
         """Get the album playing, if none it returns None """
-        return self._data.get('album') or None
+        return self.parse_data(option='album', in_information=True) or None
 
     def track_number(self):
         """Get the track_number playing, if none it returns None """
-        return self._data.get('track_number') or None
+        return self.parse_data(option='track_number', in_information=True) or None
 
     def filename(self):
         """Get the filename """
-        return self._data.get('filename') or None
+        return self.parse_data(option='filename', in_information=True) or None
 
     def title(self):
         """Get the title playing, if none it returns None """
@@ -154,7 +152,7 @@ class HttpVLC():
 
     def subtitle_delay(self):
         """Return the set delay for subtitles """
-        return self._data['subtitledelay']
+        return self.parse_data(option='subtitledelay')
 
     def set_volume(self, volume):
         """Set volume level, range 0..1."""
@@ -191,4 +189,34 @@ class HttpVLC():
 
     def play(self):
         """Start playing media playback"""
-        return self.parse_data(command="pl_play")
\ No newline at end of file
+        return self.parse_data(command="pl_play")
+
+    def play_playlist_item(self, id=0):
+        """Start playing a specific item from a playlist"""
+        return self.parse_data(command=f"pl_play&id={id}")
+
+    def delete_playlist_item(self, id=0):
+        """Delete a specific item from a playlist"""
+        return self.parse_data(command=f"pl_delete&id={id}")
+
+    def seek(self, val):
+        """Seek to a part of a video."""
+        return self.parse_data(command=f"pl_play?{val}")
+
+    def sub_delay(self, val):
+        """Select the subtitle delay for a video."""
+        return self.parse_data(command=f"subdelay?{val}")
+
+    def subtitle_track(self, val):
+        """Select the subtitle track for a video."""
+        return self.parse_data(command=f"subtitle_track?{val}")
+
+    def video_track(self, val):
+        """Select the video track for a particular media file."""
+        return self.parse_data(command=f"video_track?{val}")
+
+    def set_rate(self, val):
+        """Set the rate for a current media file"""
+        if val > 0:
+            return self.command(f'rate?{val}')
+        raise Exception('The rate must be grater than zero.')
